//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.access.api_1_svw_redbs_bss;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.access.*;
import sim.toolkit.*;
import javacard.framework.*;

import sim.test.util.*;
import sim.test.access.*;


public class API_1_SVW_REDBS_BSS_1 extends TestToolkitApplet {

    static SIMView theSimView;

    final static short RESP_BUFFER_LENGTH = (short)20;

    static byte[] resp = new byte[RESP_BUFFER_LENGTH];
    static byte[] nullBuffer = null;

    final static byte[] RESP_TESTCASE01 = {
        (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, 
        (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55,
        (byte)0x0F, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, 
        (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF
    };

    final static byte[] RESP_TESTCASE02 = {
        (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, 
        (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55,
        (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, 
        (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55
    };



    /**
     * The Constructor registers the application for the Event.
     */
    private API_1_SVW_REDBS_BSS_1() {
        
        // Get a reference to the SIM View.
        theSimView = SIMSystem.getTheSIMView();
        
    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        API_1_SVW_REDBS_BSS_1 testApp = new API_1_SVW_REDBS_BSS_1();

        // Register the applet with the JCRE.
        testApp.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet.
        testApp.init();
    }


    /**
     * This method will be used to trigger the tests.
     * The application will be triggered by an Formatted SMS PP download event.
     * @param event 
     */
    public void processToolkit (byte event) throws ToolkitException {

        short result = 0;
        short respOffset = 0;
        short respLength = 0;
        short fileOffset = 0;

        if (event == EVENT_FORMATTED_SMS_PP_ENV) {

            /*------------------------------------------------------------------
             * TEST CASE 01: Read from a transparent file at offset 0
             *----------------------------------------------------------------*/
            try {
                theSimView.select(SIMView.FID_EF_ICCID);

                // fill response buffer with 0x55
                for (short ii=0; ii<resp.length; ii++)
                    resp[ii] = 0x55;

                fileOffset = 0;
                respOffset = 10;
                respLength = 10;

                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);

                if (result != (short)(respOffset + respLength))
                    UserException.throwIt((short)0);
                
                if (Util.arrayCompare(resp, (short)0, RESP_TESTCASE01, (short)0, RESP_BUFFER_LENGTH) != 0)
                    UserException.throwIt((short)0);

                reportTestSuccess((byte)TestConstants.TESTCASE01);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE01);
            }

            /*------------------------------------------------------------------
             * TEST CASE 02: Read from a transparent file at a given offset
             *----------------------------------------------------------------*/
            try {
                // fill response buffer with 0x55
                for (short ii = 0; ii < resp.length; ii++)
                    resp[ii] = 0x55;

                fileOffset = 5;
                respOffset = 10;
                respLength = 5;

                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);

                if (result != (short)(respOffset + respLength))
                    UserException.throwIt((short)0);
                
                if (Util.arrayCompare(resp, (short)0, RESP_TESTCASE02, (short)0, RESP_BUFFER_LENGTH) != 0)
                    UserException.throwIt((short)0);

                reportTestSuccess((byte)TestConstants.TESTCASE02);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE02);
            }

            /*------------------------------------------------------------------
             * TEST CASE 03: Errorcase, fileOffset out of file boundaries
             *----------------------------------------------------------------*/
            try {
                fileOffset = (short)-1;
                respOffset = 0;
                respLength = 10;

                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);

                reportTestFailure((byte)TestConstants.TESTCASE03);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE03, 
                        sve.getReason() == SIMViewException.OUT_OF_FILE_BOUNDARIES);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE03);
            }

            /*------------------------------------------------------------------
             * TEST CASE 04: Errorcase, fileOffset + respLength > EF length
             *----------------------------------------------------------------------------------*/
            try {
                fileOffset = 9;
                respOffset = 0;
                respLength = 2;

                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);

                reportTestFailure((byte)TestConstants.TESTCASE04);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE04, 
                        sve.getReason() == SIMViewException.OUT_OF_FILE_BOUNDARIES);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE04);
            }

            /*------------------------------------------------------------------
             * TEST CASE 05: Errorcase, resp is null
             *----------------------------------------------------------------------------------*/
            try {
                fileOffset = 0;
                respOffset = 0;
                respLength = 10;
                result = theSimView.readBinary(fileOffset, nullBuffer, respOffset, respLength);
                reportTestFailure((byte)TestConstants.TESTCASE05);
            } catch (NullPointerException npe) {
                reportTestSuccess((byte)TestConstants.TESTCASE05);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE05);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 06: Errorcase, respOffset < 0
             *----------------------------------------------------------------------------------*/
            try {
                fileOffset = 0;
                respOffset = (short)-1;
                respLength = 10;
                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);
                reportTestFailure((byte)TestConstants.TESTCASE06);
            } catch (ArrayIndexOutOfBoundsException e) {
                reportTestSuccess((byte)TestConstants.TESTCASE06);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE06);
            }

            /*------------------------------------------------------------------
             * TEST CASE 07: Errorcase, respLength < 0
             *----------------------------------------------------------------------------------*/
            try {
                fileOffset = 0;
                respOffset = 0;
                respLength = (short)-1;
                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);
                reportTestFailure((byte)TestConstants.TESTCASE07);
            } catch (ArrayIndexOutOfBoundsException e) {
                reportTestSuccess((byte)TestConstants.TESTCASE07);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE07);
            }

            /*------------------------------------------------------------------
             * TEST CASE 08: Errorcase, respOffset + respLength > resp.Length
             *----------------------------------------------------------------------------------*/
            try {
                fileOffset = 0;
                respOffset = 10;
                respLength = 11;
                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);
                reportTestFailure((byte)TestConstants.TESTCASE08);
            } catch (ArrayIndexOutOfBoundsException e) {
                reportTestSuccess((byte)TestConstants.TESTCASE08);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE08);
            }

            /*-----------------------------------------------------------------------------------
             * TEST CASE 09: Errorcase, EF is not transparent
             *----------------------------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_LARU);

                fileOffset = 0;
                respOffset = 0;
                respLength = 1;

                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);

                reportTestFailure((byte)TestConstants.TESTCASE09);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE09, 
                        sve.getReason() == SIMViewException.FILE_INCONSISTENT);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE09);
            }

            /*-----------------------------------------------------------------------------------
             * TEST CASE 10: Errorcase, Access condition not fulfilled
             *----------------------------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_TNR);

                fileOffset = 0;
                respOffset = 0;
                respLength = 1;

                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);

                reportTestFailure((byte)TestConstants.TESTCASE10);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE10, 
                        sve.getReason() == SIMViewException.AC_NOT_FULFILLED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE10);
            }

            /*-----------------------------------------------------------------------------------
             * TEST CASE 11: Errorcase, EF is invalidated
             *----------------------------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_TNU);

                theSimView.invalidate();

                try {
                    result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALIDATION_STATUS_CONTRADICTION)
                        UserException.throwIt((short)0);
                } finally {
                    theSimView.rehabilitate();
                }

                reportTestSuccess((byte)TestConstants.TESTCASE11);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE11);
            }

            /*-----------------------------------------------------------------------------------
             * TEST CASE 12: Errorcase, No EF selected
             *----------------------------------------------------------------------------------*/
            try {
                theSimView.select(SIMView.FID_MF);

                result = theSimView.readBinary(fileOffset, resp, respOffset, respLength);

                reportTestFailure((byte)TestConstants.TESTCASE12);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE12, 
                        sve.getReason() == SIMViewException.NO_EF_SELECTED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE12);
            }
        
        } // end if
    }
}
