//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.access.api_1_svw_updbs_bss;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.access.*;
import sim.toolkit.*;
import javacard.framework.*;

import sim.test.util.*;
import sim.test.access.*;


public class API_1_SVW_UPDBS_BSS_1 extends TestToolkitApplet {

    static SIMView theSimView;

    final static short RESP_BUFFER_LENGTH = (short)20;
    final static short DATA_BUFFER_LENGTH = (short)20;

    static byte[] data = new byte[DATA_BUFFER_LENGTH];
    static byte[] resp = new byte[RESP_BUFFER_LENGTH];
    static byte[] nullBuffer = null;

    /**
     * The Constructor registers the application for the Event.
     */
    private API_1_SVW_UPDBS_BSS_1() {
        
        // Get a reference to the SIM View.
        theSimView = SIMSystem.getTheSIMView();

    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        API_1_SVW_UPDBS_BSS_1 testApp = new API_1_SVW_UPDBS_BSS_1();

        // Register the applet with the JCRE.
        testApp.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet.
        testApp.init();
    }

    /**
     * This method will be used to trigger the tests.
     * The application will be triggered by an Formatted SMS PP download event.
     * @param event 
     */
    public void processToolkit (byte event) throws ToolkitException {

        short result = 0;
        short fileOffset = 0;
        short dataOffset = 0;
        short dataLength = 0;
        short respOffset = 0;
        short respLength = 0;

        if (event == EVENT_FORMATTED_SMS_PP_ENV) {

            /*------------------------------------------------------------------
             * TEST CASE 01: No EF selected
             *----------------------------------------------------------------*/
            try {
                fileOffset = 0;
                data[0] = 0x55;
                dataOffset = 0;
                dataLength = 10;

                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE01);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE01, 
                        sve.getReason() == SIMViewException.NO_EF_SELECTED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE01);
            }

            /*------------------------------------------------------------------
             * TEST CASE 02: Update Transparent EF
             *----------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_TARU);

                fileOffset = 3;
                data[0] = (byte)0x55;
                dataOffset = 0;
                dataLength = 1;
                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);

                fileOffset = 3;
                respOffset = 0;
                respLength = 1;
                result = theSimView.readBinary(fileOffset, resp, respOffset, 
                                                respLength);

                if (resp[0] != (byte)0x55) {
                    UserException.throwIt((short)0);
                }
                reportTestSuccess((byte)TestConstants.TESTCASE02);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE02);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 03: Update Transparent EF
             *----------------------------------------------------------------*/
            try {
                fileOffset = 254;
                data[0] = (byte)0x55;
                data[1] = (byte)0xAA;
                data[2] = (byte)0x66;
                dataOffset = 0;
                dataLength = 3;
                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);

                fileOffset = 254;
                respOffset = 0;
                respLength = 3;
                result = theSimView.readBinary(fileOffset, resp, respOffset, 
                                                respLength);

                if ((resp[0] != (byte)0x55) || (resp[1] != (byte)0xAA) || 
                    (resp[2] != (byte)0x66)) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE03);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE03);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 04: Offset into File out of bounds
             *----------------------------------------------------------------*/
            try {
                fileOffset = (short)-1;
                dataOffset = 0;
                dataLength = 10;
                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE04);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE04, 
                        sve.getReason() == SIMViewException.OUT_OF_FILE_BOUNDARIES);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE04);
            }
        
            /*-----------------------------------------------------------------
             * TEST CASE 05: fileOffset + dataLength > EF length
             *----------------------------------------------------------------*/
            try {
                fileOffset = 259;
                dataOffset = 0;
                dataLength = 2;

                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE05);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE05, 
                        sve.getReason() == SIMViewException.OUT_OF_FILE_BOUNDARIES);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE05);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 06: data is null
             *----------------------------------------------------------------*/
            try {
                fileOffset = 0;
                dataOffset = 0;
                dataLength = 10;
                theSimView.updateBinary(fileOffset, nullBuffer, dataOffset, dataLength);
                reportTestFailure((byte)TestConstants.TESTCASE06);
            } catch (NullPointerException npe) {
                reportTestSuccess((byte)TestConstants.TESTCASE06);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE06);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 07: dataOffset < 0
             *----------------------------------------------------------------*/
            try {
                fileOffset = 0;
                dataOffset = (short)-1;
                dataLength = 10;
                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);
                reportTestFailure((byte)TestConstants.TESTCASE07);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE07);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE07);
            }

            /*------------------------------------------------------------------
             * TEST CASE 08: dataLength < 0
             *----------------------------------------------------------------*/
            try {
                fileOffset = 0;
                dataOffset = 0;
                dataLength = (short)-1;
                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);
                reportTestFailure((byte)TestConstants.TESTCASE08);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE08);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE08);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 09: dataOffset + dataLength > data.length
             *----------------------------------------------------------------*/
            try {
                fileOffset = 0;
                dataOffset = 10;
                dataLength = 11;
                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);
                reportTestFailure((byte)TestConstants.TESTCASE09);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE09);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE09);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 10: EF is not Transparent
             *----------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_LARU);

                fileOffset = 0;
                data[0] = 0x55;
                dataOffset = 0;
                dataLength = 1;

                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE10);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE10, 
                        sve.getReason() == SIMViewException.FILE_INCONSISTENT);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE10);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 11: Access condition not fulfilled
             *----------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_TNU);

                fileOffset = 0;
                data[0] = (byte)0x55;
                dataOffset = 0;
                dataLength = 1;

                theSimView.updateBinary(fileOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE11);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE11, 
                        sve.getReason() == SIMViewException.AC_NOT_FULFILLED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE11);
            }
        
            /*------------------------------------------------------------------
             * TEST CASE 12: EF is invalidated
             *----------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_TNR);

                theSimView.invalidate();

                fileOffset = 0;
                dataOffset = 0;
                dataLength = 1;
                try {
                    theSimView.updateBinary(fileOffset, data, dataOffset, 
                                            dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALIDATION_STATUS_CONTRADICTION)
                        UserException.throwIt((short)0);
                } finally {
                    theSimView.rehabilitate();
                }

                reportTestSuccess((byte)TestConstants.TESTCASE12);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE12);
            }
        
        } // end if
    }
}
