//-----------------------------------------------------------------------------
//    API_2_TKR_RTIMB_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_tkr_rtimb;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.* ;
import javacard.framework.* ;
import sim.test.util.* ;

/**
 * Test Area : sim.toolkit.ToolkitRegistry.releaseTimer(byte timerID) method
 * 
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 * @version 0.0.1 - 15/01/01
 *
 */
public class API_2_TKR_RTIMB_1 extends TestToolkitApplet
{

	// -----------------------------------------------------------------
	// Applet was installed with max timer parameter equals to 8
	// -----------------------------------------------------------------

	//Timer ID value range
	private	static short MIN_TIMER_ID = 1;
	private	static short MAX_TIMER_ID = 8;
	
    /**
     * Constructor of the applet
     */
    public API_2_TKR_RTIMB_1() 
    {
    }

    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) 
    {
        // Create a new applet instance
        API_2_TKR_RTIMB_1 thisApplet = new API_2_TKR_RTIMB_1();

        // Register the new applet instance to the JCRE            
        //thisApplet.register();
		thisApplet.register(bArray, (short)(bOffset+1), (byte) bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();    
    }
    

    /**
       * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) 
    {
        // Result of tests
        boolean bRes ;

        // Number of tests
        byte testCaseNb = (byte) 0x07;
        
        // Get the system instance of the ProactiveHandler class
        ProactiveHandler proHdlr = ProactiveHandler.getTheHandler() ;

        ProactiveResponseHandler proRespHdlr = null ;
 
		if (event == EVENT_FORMATTED_SMS_PP_ENV) {

			// -----------------------------------------------------------------
			// Test Case 1 :Releasing not allocated timers
			//				For each timer ID ranging from 00 to FF, 
			//				applet calls releaseTimer(ID)
			//				Each time, method shall throw a ToolkitException 
			//				with reason code INVALID_TIMER_ID
			// -----------------------------------------------------------------
        
			testCaseNb = (byte) 1 ;
			bRes = false;
		
			try{
				obReg.releaseTimer((byte) 0);
			}
			catch (ToolkitException e) {	//INVALID_TIMER_ID ToolkitException 
											//shall be thrown
					bRes = (e.getReason()==ToolkitException.INVALID_TIMER_ID);
			}
			catch (Exception e) {
				bRes = false;
			}

			try{
				for (short i = ((short) 1); (i < ((short) 256)) && (bRes);i++){
					bRes = false;
					obReg.releaseTimer((byte) i);
				}
			}
			catch (ToolkitException e) {	//INVALID_TIMER_ID ToolkitException 
											//shall be thrown
					bRes = (e.getReason()==ToolkitException.INVALID_TIMER_ID);
			}
			catch (Exception e) {
				bRes = false;
			}

			reportTestOutcome(testCaseNb, bRes) ;


			// -----------------------------------------------------------------
			// Test Case 2 :Releasing allocated timers
			//				1 - allocateTimer() for all the possible timers
			//					No exception shall be thrown
			//
			//				2 - For all the possible timers except the last one :
			//					2.1 -	releaseTimer(id)
			//							Each time, no exception shall be thrown
			//					2.2 -	isEventSet(EVENT_TIMER_EXPIRATION) 
			//							Each time shall return true
			// -----------------------------------------------------------------

			testCaseNb = (byte) 2 ;
			bRes = false;
	
			// -----------------------------------------------------------------
			//				1 - allocateTimer() for all the possible timers
			//					No exception shall be thrown
			// -----------------------------------------------------------------
			try{
					for (short i = MIN_TIMER_ID; (i < (short) (MAX_TIMER_ID + 1));
							i++){
						obReg.allocateTimer();
					}
					bRes = true;
			}
			catch (Exception e) {	//No Exception shall be thrown
				bRes = false;
			}

			// -----------------------------------------------------------------
			//				2 - For all the possible timers except the last one :
			// -----------------------------------------------------------------

			// -----------------------------------------------------------------
			//					2.1 -	releaseTimer(id)
			//							Each time, no exception shall be thrown
			// -----------------------------------------------------------------
			try{
				obReg.releaseTimer((byte) MIN_TIMER_ID);

			// -----------------------------------------------------------------
			//					2.2 -	isEventSet(EVENT_TIMER_EXPIRATION) 
			//							Each time shall return true
			// -----------------------------------------------------------------
				bRes = bRes && obReg.isEventSet(EVENT_TIMER_EXPIRATION);
			}
			catch (Exception e) {	//No Exception shall be thrown
				bRes = false;
			}


			try{
					for (short i = (short) (MIN_TIMER_ID + 1); 
						(i < MAX_TIMER_ID ) && (bRes);i++){

			// -----------------------------------------------------------------
			//					2.1 -	releaseTimer(id)
			//							Each time, no exception shall be thrown
			// -----------------------------------------------------------------
						obReg.releaseTimer((byte) i);

			// -----------------------------------------------------------------
			//					2.2 -	isEventSet(EVENT_TIMER_EXPIRATION) 
			//							Each time shall return true
			// -----------------------------------------------------------------
						bRes = obReg.isEventSet(EVENT_TIMER_EXPIRATION);
					}
			}
			catch (Exception e) {	//No Exception shall be thrown
				bRes = false;
			}

			reportTestOutcome(testCaseNb, bRes) ;

			// -----------------------------------------------------------------
			// Test Case 3 :Releasing invalid timer ID
			//				1 - releaseTimer(FF) method
			//					Shall throw a ToolkitException 
			//					with INVALID_TIMER_ID reason code
			//
			//				2 - isEventSet(EVENT_TIMER_EXPIRATION)
			//					Shall return true
			// -----------------------------------------------------------------

			testCaseNb = (byte) 3 ;
			bRes = false;
			
			// -----------------------------------------------------------------
			//				1 - releaseTimer(FF) method
			//					Shall throw a ToolkitException	
			//					with INVALID_TIMER_ID reason code
			// -----------------------------------------------------------------
			try{
				obReg.releaseTimer((byte) 255);
			}
			catch (ToolkitException e){
				bRes = (e.getReason()==ToolkitException.INVALID_TIMER_ID);	
			}
			catch (Exception e) {
				bRes = false;
			}
			
			// -----------------------------------------------------------------
			//				2 - isEventSet(EVENT_TIMER_EXPIRATION)
			//					Shall return true	
			// -----------------------------------------------------------------
			bRes = bRes && obReg.isEventSet(EVENT_TIMER_EXPIRATION);

			reportTestOutcome(testCaseNb, bRes) ;


			// -----------------------------------------------------------------
			// Test Case 4 :Releasing last timer
			//				1 - releaseTimer(last timer allocated)
			//					No exception shall be thrown
			//
			//				2 - isEventSet(EVENT_TIMER_EXPIRATION)
			//					Shall return false 
			// -----------------------------------------------------------------

			testCaseNb = (byte) 4 ;
			bRes = false;
			
			// -----------------------------------------------------------------
			//				1 - releaseTimer(08)
			//					No exception shall be thrown	
			// -----------------------------------------------------------------
			try{
				obReg.releaseTimer((byte) MAX_TIMER_ID);
				bRes = true;
			}
			catch (Exception e){
				bRes = false;	
			}
		
			// -----------------------------------------------------------------
			//				2 - isEventSet(EVENT_TIMER_EXPIRATION)
			//					Shall return false 	
			// -----------------------------------------------------------------
			bRes = bRes && !obReg.isEventSet(EVENT_TIMER_EXPIRATION);

			reportTestOutcome(testCaseNb, bRes) ;


			// -----------------------------------------------------------------
			// Test Case 5 :Checking we can allocate timers 
			//				after they have been released
			//				allocateTimer() for all the possible timers
			//				No exception shall be thrown
			// -----------------------------------------------------------------

			testCaseNb = (byte) 5 ;
			bRes = false;
	
			try{
					for (short i = MIN_TIMER_ID; (i < (short) (MAX_TIMER_ID + 1));i++){
						obReg.allocateTimer();
					}
					bRes = true;
			}
			catch (Exception e) {	//No Exception shall be thrown
				bRes = false;
			}

			reportTestOutcome(testCaseNb, bRes) ;


			// -----------------------------------------------------------------
			// Test Case 6 :Releasing all timers
			//				releaseTimer() for all the possible timers
			//				No exception shall be thrown
			// -----------------------------------------------------------------

			testCaseNb = (byte) 6 ;
			bRes = false;
	
			try{
					for (short i = MIN_TIMER_ID; (i < (short) (MAX_TIMER_ID+1));i++){
						obReg.releaseTimer((byte) i);
					}
					bRes = true;
			}
			catch (Exception e) {	//No Exception shall be thrown
				bRes = false;
			}

			reportTestOutcome(testCaseNb, bRes) ;


			// -----------------------------------------------------------------
			// Test Case 7 :Initialiation of the test case
			// -----------------------------------------------------------------

			testCaseNb = (byte) 7 ;
			bRes = true;
			reportTestOutcome(testCaseNb, bRes) ;
			
		}
		else{
		
			if (event == EVENT_TIMER_EXPIRATION) {

				// -----------------------------------------------------------------
				// Test Case 7 :Checking applet isn't triggered 
				//				by ENVELOPE(TIMER_EXPIRATION) command any more
				// -----------------------------------------------------------------

				testCaseNb = (byte) 7 ;	
				reportTestFailure(testCaseNb) ;
			}
		}
	} 
}
