//-----------------------------------------------------------------------------
//    API_2_ERH_COPY_BSS_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_erh_copy_bss;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.* ;
import javacard.framework.* ;
import sim.test.util.* ;

/**
 * Test Area : sim.toolkit.EnvelopeResponseHandler.copy(byte[] dstBuffer, short dstOffset, short dstLength) method
 * 
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 * @version 1.0.0 - 01/08/01
 * @author 3GPP T3 SWG API
 *
 * @see sim.toolkit.ProactiveResponseHandler
 */
public class API_2_ERH_COPY_BSS_1 extends TestToolkitApplet
{

    /** Registry entry concerning applet instance */
    private ToolkitRegistry oReg;

    private byte[] TEXT = {(byte)'T',(byte)'e',(byte)'x',(byte)'t'} ;

    
	// Number of tests
    byte testCaseNb = (byte) 0x00 ;
        
    /**
     * Constructor of the applet
     */
    public API_2_ERH_COPY_BSS_1() 
    {
		// Specific registration to Unrecognized Envelope
		obReg.setEvent(EVENT_UNRECOGNIZED_ENVELOPE);
    }


    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) 
    {
        // Create a new applet instance
        API_2_ERH_COPY_BSS_1 thisApplet = new API_2_ERH_COPY_BSS_1();

        // Register the new applet instance to the JCRE            
        thisApplet.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();    
    }
    

    /**
       * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) 
    {
        // Result of tests
        boolean bRes ;
        
        // Get the system instance of the EnvelopeResponseHandler class
        EnvelopeResponseHandler EnvRespHdlr = EnvelopeResponseHandler.getTheHandler() ;
        
        switch ( testCaseNb ) 
		{
		case (byte) 0 :
		 
		
        // --------------------------------------------
        // Test Case 1 : try to use copy value methode with the destination buffer null
        testCaseNb = (byte) 1 ;
        bRes = false ;
        
    
        // Null as dstBuffer
        try {
			// Build a 7Fh byte command
			byte[] SMS_TPDU_Value2=new byte[0x07];
						
			//initialize the buffer
			for( short i=0;i<(short)7;i++)
				SMS_TPDU_Value2[i]=(byte)i;
			
			EnvRespHdlr.appendTLV((byte)0x33,SMS_TPDU_Value2,(short) 0x00, (short) 0x07);
				
            EnvRespHdlr.copy(null, (short)0, (short)1) ;
        }
		catch (NullPointerException e) {
            bRes = true ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 2 : dstOffset >= dstBuffer.length
        testCaseNb = (byte) 2 ;
        bRes = false ;
        
        try {
            byte[] dstBuffer = new byte[5] ;
    
			EnvRespHdlr.copy(dstBuffer, (short)5, (short)1) ;
        }
		catch (ArrayIndexOutOfBoundsException e) {
            bRes = true ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 3 : dstOffset < 0
        testCaseNb = (byte) 3 ;
        bRes = false ;
        
        try {
            byte[] dstBuffer = new byte[5] ;
    
            EnvRespHdlr.copy(dstBuffer, (short)-1, (short)1) ;
			
        } 
		catch (ArrayIndexOutOfBoundsException e) {
            bRes = true ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 4 : dstLength > dstBuffer.length
        testCaseNb = (byte) 4 ;
        bRes = false ;
        
        try {
            byte[] dstBuffer = new byte[5] ;
    
            EnvRespHdlr.copy(dstBuffer, (short)0, (short)6) ;
        }
		catch (ArrayIndexOutOfBoundsException e) {
            bRes = true ;
        }
    
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

      
        // --------------------------------------------
        // Test Case 5 : dstOffset + dstLength > dstBuffer.length
        testCaseNb = (byte) 5 ;
        bRes = false ;
        
        try {
            byte[] dstBuffer = new byte[5] ;
            
            EnvRespHdlr.copy(dstBuffer, (short)3, (short)3) ;
        }
		catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 6 : dstLength < 0
        testCaseNb = (byte) 6 ;
        bRes = false ;
        
        try {
            byte[] dstBuffer = new byte[5] ;
            
            EnvRespHdlr.copy(dstBuffer, (short)0, (short)-1) ;
        }
		catch (ArrayIndexOutOfBoundsException e) {
            bRes = true ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 7 : dstLength > length of the simple TLV list
        testCaseNb = (byte) 7 ;
        bRes = false ;
        
        try {
			byte[] dstBuffer = new byte[10] ;
        
            EnvRespHdlr.copy(dstBuffer, (short)0, (short)10) ;
        }
		catch (ToolkitException e) {
            bRes = (e.getReason() == ToolkitException.OUT_OF_TLV_BOUNDARIES) ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 8 : Successful call, whole buffer
        testCaseNb = (byte) 8 ;
        bRes = false ;
        
        try {
			byte[] dstBuffer = new byte[9] ;
        
			if(EnvRespHdlr.copy(dstBuffer, (short)0, (short)9)==9)
				bRes=true;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;
		
        
        // --------------------------------------------
        // Test Case 9 : Compare the buffer
        testCaseNb = (byte) 9 ;
        bRes = false ;
        
    	byte[] compareBuffer = new byte[9] ;
    
		try {
            
	
			// Initialise buffers
			for (short i=0; i<(short)7; i++)
				compareBuffer[(short)(i+2)] = (byte)i ;
		
			compareBuffer[0]=0x33;
			compareBuffer[1]=0x07;
		
            // Compare buffers
            byte[] dstBuffer = new byte[9] ;
        

			if((EnvRespHdlr.copy(dstBuffer, (short)0, (short)9)==9)&&
				(Util.arrayCompare(compareBuffer, (short)0,dstBuffer, (short)0, (short)9)==0))
					bRes=true;
		} 
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;
		return;
       
	case (byte) 9 :
		
		// --------------------------------------------
        // Test Case 10 : Successful call, part of a buffer
        testCaseNb = (byte) 10 ;
        bRes = false ;
        
        
		// Compare buffers
        compareBuffer = new byte[15] ;
		byte[] dstBuffer = new byte[15] ;
        
		// Build a 7Fh byte command
		byte[] SMS_TPDU_Value2=new byte[13];
						
		//initialize the buffer
		for( short i=0;i<(short)13;i++)
			SMS_TPDU_Value2[i]=(byte)i;
			
		EnvRespHdlr.appendTLV((byte)0x33,SMS_TPDU_Value2,(short) 0x00, (short) 13);
			
		try {
			
			if(EnvRespHdlr.copy(dstBuffer, (short)3, (short)9)==12)
				bRes=true;
		}
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 11 : Compare the buffer
        testCaseNb = (byte) 11 ;
        bRes = false ;
        
        dstBuffer = new byte[15] ;
        
		for( short i=5;i<(short)12;i++)
			compareBuffer[i]=(byte)(i-5);
	
		
		
		compareBuffer[3]=0x33;
		compareBuffer[4]=0x0D;
		
		try {
			
			if((EnvRespHdlr.copy(dstBuffer, (short)3, (short)9)==12)&&
			(Util.arrayCompare(compareBuffer, (short)0,dstBuffer, (short)0, (short)15)==0))
				bRes=true;
		}
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

		return;
       
	case (byte) 11 :
	
        // --------------------------------------------
        // Test Case 12 : Successful call, part of a buffer
        testCaseNb = (byte) 12 ;
        bRes = false ;
        
        
		// Compare buffers
        compareBuffer = new byte[15] ;
    
		dstBuffer = new byte[15] ;
        
		// Build a 7Fh byte command
		SMS_TPDU_Value2=new byte[13];
						
		//initialize the buffer
		for( short i=0;i<(short)13;i++)
			SMS_TPDU_Value2[i]=(byte)i;				
		
		EnvRespHdlr.appendTLV((byte)0x33,SMS_TPDU_Value2,(short) 0x00, (short) 13);
		
		
		try {
			
			if(EnvRespHdlr.copy(dstBuffer, (short)3, (short)6)==9)
				bRes=true;
		}
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 13 : Compare the buffer
        testCaseNb = (byte) 13 ;
        bRes = false ;
        
        dstBuffer = new byte[15] ;
        
		for( short i=5;i<(short)9;i++)
			compareBuffer[i]=(byte)(i-5);
			
		compareBuffer[3]=0x33;
		compareBuffer[4]=0x0D;
		Util.arrayCopy(compareBuffer, (short)0,compareBuffer, (short)0, (short)15);

		try {
			if((EnvRespHdlr.copy(dstBuffer, (short)3, (short)6)==9)&&
			(Util.arrayCompare(compareBuffer, (short)0,dstBuffer, (short)0, (short)15)==0))
				bRes=true;
		}
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 14 : successful call with the length=0 and the offset = length of the destination buffer
        testCaseNb = (byte) 14 ;
        bRes = false ;
        
        try {
			if(EnvRespHdlr.copy(dstBuffer, (short)dstBuffer.length, (short)0)==15)
				bRes=true;
		}
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


		}
    }
}
